@@ -50,25 +50,8 @@ let generate_error resp =
   resp # send `Resp_end;
 ;;
 
-class config 
-        ?(max_reqline_length = 256)
-        ?(max_header_length = 32768)
-        ?(max_trailer_length = 32768)
-        ?(limit_pipeline_length = 5)
-        ?(limit_pipeline_size = max_int)
-        () : Nethttpd_kernel.http_protocol_config =
-object
-  method config_max_reqline_length = max_reqline_length
-  method config_max_header_length = max_header_length
-  method config_max_trailer_length = max_trailer_length
-  method config_limit_pipeline_length = limit_pipeline_length
-  method config_limit_pipeline_size = limit_pipeline_size
-  method config_announce_server = `Ocamlnet
-end
-;;
-
 let serve fd =
-  let config = new config() in
+  let config = Nethttpd_kernel.default_http_protocol_config in
   let proto = new Nethttpd_kernel.http_protocol config fd in
 
   let rec next_token () =
@@ -156,5 +139,31 @@ let start() =
 ;;
 
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+let conf_debug() =
+  (* Set the environment variable DEBUG to either:
+       - a list of Netlog module names
+       - the keyword "ALL" to output all messages
+       - the keyword "LIST" to output a list of modules
+     By setting DEBUG_WIN32 additional debugging for Win32 is enabled.
+   *)
+  let debug = try Sys.getenv "DEBUG" with Not_found -> "" in
+  if debug = "ALL" then
+    Netlog.Debug.enable_all()
+  else if debug = "LIST" then (
+    List.iter print_endline (Netlog.Debug.names());
+    exit 0
+  )
+  else (
+    let l = Netstring_str.split (Netstring_str.regexp "[ \t\r\n]+") debug in
+    List.iter
+      (fun m -> Netlog.Debug.enable_module m)
+      l
+  );
+  if (try ignore(Sys.getenv "DEBUG_WIN32"); true with Not_found -> false) then
+    Netsys_win32.Debug.debug_c_wrapper true
+;;
+
+
+Netsys_signal.init();
+conf_debug();
 start();;
