@@ -3,8 +3,6 @@
  *)
 
 open Netcgi;;
-open Netcgi_types;;
-open Netcgi_env;;
 open Nethttpd_reactor;;
 open Printf;;
 
@@ -17,9 +15,12 @@ let rec service_loop reactor netcgi_processor =
 	    let env =
 	      req # environment in
 	    let cgi = 
-	      new std_activation 
-		   ~env:(env :> cgi_environment) 
-		   ~operating_type:buffered_transactional_optype () in
+	      Netcgi_common.cgi_with_args 
+		(new Netcgi_common.cgi)
+		(env :> Netcgi.cgi_environment)
+		Netcgi.buffered_transactional_outtype
+		env#input_channel
+		(fun _ _ _ -> `Automatic) in
 	    netcgi_processor cgi
 	  with
 	      e ->
@@ -35,22 +36,7 @@ let rec service_loop reactor netcgi_processor =
 
 
 let serve fd netcgi_processor =
-  let config : http_reactor_config = 
-    object
-      method config_timeout_next_request = 15.0
-      method config_timeout = 300.0
-      method config_reactor_synch = `Write
-      method config_cgi = Netcgi_env.default_config 
-      method config_error_response n = "<html>Error " ^ string_of_int n ^ "</html>"
-      method config_log_error _ _ _ _ msg = 
-	printf "Error log: %s\n" msg
-      method config_max_reqline_length = 256
-      method config_max_header_length = 32768
-      method config_max_trailer_length = 32768
-      method config_limit_pipeline_length = 5
-      method config_limit_pipeline_size = 250000
-      method config_announce_server = `Ocamlnet
-    end in
+  let config = Nethttpd_reactor.default_http_reactor_config in
   let reactor = new http_reactor config fd in
   service_loop reactor netcgi_processor;
   reactor # close()
@@ -245,6 +231,30 @@ let process (cgi : cgi_activation) =
 	cgi # output # commit_work()
 ;;
 
+let conf_debug() =
+  (* Set the environment variable DEBUG to either:
+       - a list of Netlog module names
+       - the keyword "ALL" to output all messages
+       - the keyword "LIST" to output a list of modules
+     By setting DEBUG_WIN32 additional debugging for Win32 is enabled.
+   *)
+  let debug = try Sys.getenv "DEBUG" with Not_found -> "" in
+  if debug = "ALL" then
+    Netlog.Debug.enable_all()
+  else if debug = "LIST" then (
+    List.iter print_endline (Netlog.Debug.names());
+    exit 0
+  )
+  else (
+    let l = Netstring_str.split (Netstring_str.regexp "[ \t\r\n]+") debug in
+    List.iter
+      (fun m -> Netlog.Debug.enable_module m)
+      l
+  );
+  if (try ignore(Sys.getenv "DEBUG_WIN32"); true with Not_found -> false) then
+    Netsys_win32.Debug.debug_c_wrapper true
+;;
 
 (* main: *)
+conf_debug();
 start process ;;
