@@ -1,17 +1,17 @@
-(* $Id: netmcore_sem.ml 1755 2012-03-24 20:18:30Z gerd $ *)
+(* $Id$ *)
 
 open Netmcore_heap
 
-type semaphore = Netmcore.res_id * string
+type semaphore = Netmcore.res_id * Bytes.t
     (* An empty string is a dummy sem.  Otherwise, the string is on
        the heap, and it contains the semaphore value *)
 
-let dummy() = (`Resource(-1),"")
+let dummy() = (`Resource(-1), Bytes.create 0)
 
 let create mut initval =
   let pool = Netmcore_heap.mut_pool mut in
   let c = Netmcore_heap.mut_sem_container mut in
-  let s = String.create Netsys_sem.sem_size in
+  let s = Bytes.create Netsys_sem.sem_size in
   let sem = (pool,s) in
   let sem_heap = add mut sem in
   let s_addr = Netsys_mem.obj_address (Obj.repr (snd sem_heap)) in
@@ -21,7 +21,7 @@ let create mut initval =
 
 let get_sem sem_heap =
   let (pool, s_heap) = sem_heap in
-  if s_heap = "" then
+  if Bytes.length s_heap = 0 then
     failwith "Netmcore_sem: cannot access dummy semaphore";
   let s_addr = Netsys_mem.obj_address (Obj.repr s_heap) in
   let m = Netsys_mem.grab s_addr Netsys_sem.sem_size in
@@ -30,7 +30,7 @@ let get_sem sem_heap =
 
 let destroy sem_heap =
   let (pool, s_heap) = sem_heap in
-  if s_heap <> "" then (
+  if Bytes.length s_heap > 0 then (
     let sem = get_sem sem_heap in
     let c = Netmcore_mempool.sem_container pool in
     Netsys_sem.sem_destroy c sem
