@@ -1,4 +1,4 @@
-(* $Id: netplex_types.mli 1845 2013-05-27 14:09:37Z gerd $ *)
+(* $Id$ *)
 
 (** Types for [Netplex] *)
 
@@ -121,6 +121,7 @@ type extended_address =
     | `W32_pipe of string
     | `W32_pipe_file of string
     | `Container of string * string * string * [ thread_sys_id | `Any ]
+    | `Internal of string
     ]
   (** Possible addresses:
        - [`Socket s]: The socket at this socket address
@@ -138,8 +139,63 @@ type extended_address =
          a socket or named pipe. If any container of a service is meant,
          the value [`Any] is substituted as a placeholder for a not yet
          known [thread_id].
+       - [`Internal name]: an internal service called [name]. Internal sockets
+         are only available for multithreaded programs, and are registered
+         at {!Netplex_internal}.
    *)
 
+(** Equality witness *)
+type (_,_) eq =
+  | Equal : ('a,'a) eq
+  | Not_equal
+
+(** List possible argument types for polysockets ({!Netsys_polysocket}),
+    which are the basis for internal services. Since OCaml-4.02 this
+    type is extensible by the user.
+ *)
+#ifdef HAVE_EXTENSIBLE_VARIANTS
+type _ polysocket_kind = ..
+type _ polysocket_kind +=
+#else
+type _ polysocket_kind =
+#endif
+   | Txdr : Netxdr.xdr_value polysocket_kind
+   | Tstring : string polysocket_kind
+
+
+(** Helper type for a polymorphic check whether a kind this the expected
+    kind *)
+type 'a kind_check =
+  { kind_check : 'b . 'b polysocket_kind ->  ('a,'b) eq
+  }
+
+(** Boxed version of [polysocket_kind] where the type parameter is hidden *)
+type polysocket_kind_box =
+  | Polysocket_kind_box : _ polysocket_kind -> polysocket_kind_box
+
+
+(** This type pairs a [polysocket_kind] with a [polyserver] as GADT. The
+    type parameter is hidden.
+ *)
+type polyserver_box =
+  | Polyserver_box : 'a polysocket_kind * 
+                     'a Netsys_polysocket.polyserver ->
+                        polyserver_box
+
+(** This type pairs a [polysocket_kind] with a [polyclient]
+    as GADT. The type parameter is hidden.
+ *)
+type polyclient_box =
+  | Polyclient_box : 'a polysocket_kind * 
+                     'a Netsys_polysocket.polyclient ->
+                        polyclient_box
+
+(** Internally used *)
+type extfd =
+  | OS_descr of Unix.file_descr
+  | Poly_endpoint of Netxdr.xdr_value Netsys_polysocket.polyendpoint
+
+
 (** The controller is the object in the Netplex master process/thread
     that manages the containers, logging, and service definitions
  *)
@@ -304,6 +360,11 @@ object
       * here.
      *)
 
+  method internal_sockets : (string * polyserver_box) list
+    (** The internal sockets for internal services: pairs of
+        ([protocol,server])
+     *)
+
   method socket_service_config : socket_service_config
     (** The configuration *)
 
@@ -381,6 +442,10 @@ object
     (** Whether to set the keep-alive socket option *)
   method tcp_nodelay : bool
     (** Whether to set the TCP_NODELAY option *)
+  method local_chmod : int option
+    (** Whether to chmod Unix Domain sockets *)
+  method local_chown : (int * int) option
+    (** Whether to chown (user,group) Unix Domain sockets *)
   method configure_slave_socket : Unix.file_descr -> unit
     (** A user-supplied function to configure slave sockets (after [accept]).
       * The function is called from the process/thread of the container.
@@ -498,6 +563,24 @@ object
       * the arguments.
      *)
 
+  method config_internal : (string * polysocket_kind_box) list
+    (** For internal services, this list configures which message kind
+        is used for which protocol.
+     *)
+
+  method process_internal : 
+           when_done:(unit -> unit) ->
+           container -> polyserver_box -> string -> unit
+    (** [process_internal ~when_done cont client protocol]: This function
+        is called instead of [process] when a connection to an internal
+        service is made. This method has to accept or reject the connection
+        with {!Netsys_polysocket.accept} or {!Netsys_polysocket.refuse},
+        respectively. The default is to refuse.
+
+        Like [process], the function must call [when_done] when the connection
+        is fully processed.
+     *)
+
   method system_shutdown : unit -> unit
     (** A user-supplied function that is called when a system shutdown
       * notification arrives. This notification is just for information
@@ -585,7 +668,7 @@ object
   method event_system : Unixqueue.unix_event_system
     (** The event system the container uses *)
 
-  method start : Unix.file_descr -> Unix.file_descr -> unit
+  method start : extfd -> extfd -> unit
     (** {b Internal Method.} Called by the controller to start the container.
       * It is the responsibility of the container to call the 
       * [post_start_hook] and the [pre_finish_hook].
@@ -666,7 +749,7 @@ object
   method set_var : string -> param_value_or_any -> unit
     (** Sets the value of a container variable *)
 
-  method call_plugin : plugin -> string -> Xdr.xdr_value-> Xdr.xdr_value
+  method call_plugin : plugin -> string -> Netxdr.xdr_value-> Netxdr.xdr_value
     (** [call_plugin p procname procarg]: This method can be called
         from the container context to invoke the plugin [p] procedure
         [procname]. This means that the [ctrl_receive_call] of the
@@ -738,8 +821,8 @@ object
     (** The plugin has been unplugged from this controller *)
 
   method ctrl_receive_call : 
-            controller -> container_id -> string -> Xdr.xdr_value -> 
-            (Xdr.xdr_value option -> unit) ->
+            controller -> container_id -> string -> Netxdr.xdr_value -> 
+            (Netxdr.xdr_value option -> unit) ->
               unit
     (** [ctrl_receive_call ctrl cid procname procarg emit]:
         This method is called in the controller context [ctrl] when a procedure
