@@ -845,6 +845,23 @@ module Header = struct
       | Some(QString v) -> Stream.junk stream; v
       | _ -> raise Stream.Failure
 
+  let rec parse_eq_suffix stream =
+    match Stream.peek stream with
+      | Some (Special '=') ->
+          Stream.junk stream;
+          "=" ^ parse_eq_suffix stream
+      | _ ->
+          ""
+  let parse_token68_or_qstring stream =
+    (* token68: see RFC 7235 *)
+    match Stream.peek stream with
+      | Some(Atom tok) ->
+          Stream.junk stream;
+          let suff = parse_eq_suffix stream in
+          tok ^ suff
+      | Some(QString v) -> Stream.junk stream; v
+      | _ -> raise Stream.Failure
+
   let rec parse_params stream =
     match Stream.npeek 3 stream with
       | [ Special ';'; Atom name; Special '=' ] ->
@@ -901,6 +918,11 @@ module Header = struct
     with
 	Not_found -> s
 
+  let print_param_value p_val =
+    match p_val with
+      | `Q s -> s
+      | `V s -> string_of_value s 
+
   let string_of_params l =
     if l = [] then
       ""
@@ -1692,7 +1714,7 @@ module Header = struct
         | [ Atom ap_name; Special '=' ] ->
             Stream.junk stream;
             Stream.junk stream;
-            let ap_val = parse_token_or_qstring stream in
+            let ap_val = parse_token68_or_qstring stream in
             let rest = parse_auth_param_rest stream in
 	    (ap_name, ap_val) :: rest
         | _ ->
@@ -1704,7 +1726,7 @@ module Header = struct
             Stream.junk stream;
             Stream.junk stream;
             Stream.junk stream;
-            let ap_val = parse_token_or_qstring stream in
+            let ap_val = parse_token68_or_qstring stream in
 	    let rest = parse_auth_param_rest stream in
 	    (ap_name, ap_val) :: rest
 	| _ ->
@@ -1746,11 +1768,6 @@ module Header = struct
     in
     parse_comma_separated_field mh fn_name parse_challenge fieldname
 
-  let encode_param p_val =
-    match p_val with
-      | `Q s -> s
-      | `V s -> string_of_value s 
-                                
   let mk_challenges fields =
     String.concat "," 
       (List.map
@@ -1759,7 +1776,7 @@ module Header = struct
               match STRING_LOWERCASE auth_name with
                 | "negotiate" ->
                      ( match auth_params with
-                         | [ "credentials", data ] -> encode_param data
+                         | [ "credentials", data ] -> print_param_value data
                          | _ -> ""
                      )
                 | _ ->
@@ -1767,7 +1784,7 @@ module Header = struct
                         ","
 		        (List.map
 		           (fun (p_name, p_val) ->
-		              p_name ^ "=" ^ encode_param p_val
+                              p_name ^ "=" ^ print_param_value p_val
                            )
 		           auth_params
                         )
@@ -1807,8 +1824,18 @@ module Header = struct
     with
       | Bad_header_field _ -> failwith "parse_quoted_parameters"
 
+  let value_of_param =
+    function
+    | `V s -> s
+    | `Q s ->
+        (match parse_quoted_parameters ("x=" ^ s) with
+           | [ _, u ] -> u
+           | _ -> raise Not_found
+        )
+
   let set_www_authenticate mh fields =
-    mh # update_field "WWW-Authenticate" (mk_challenges fields)
+    let mfields = List.map (fun field -> mk_challenges [field]) fields in
+    mh # update_multiple_field "WWW-Authenticate" mfields
 
   let get_proxy_authenticate mh =
     mark_many_decoded
@@ -1819,6 +1846,29 @@ module Header = struct
 
   let ws_re = Netstring_str.regexp "[ \t\r\n]+";;
 
+  let rec parse_auth_params stream =
+    match Stream.npeek 2 stream with
+      | [ Atom ap_name; Special '=' ] ->
+          Stream.junk stream;
+          Stream.junk stream;
+          let ap_val = parse_token68_or_qstring stream in
+          let rest = parse_auth_param_rest stream in
+          (ap_name, ap_val) :: rest
+      | _ ->
+          raise Stream.Failure
+
+  and parse_auth_param_rest stream =
+    match Stream.npeek 3 stream with
+      | [ Special ','; Atom ap_name; Special '=' ] ->
+          Stream.junk stream;
+          Stream.junk stream;
+          Stream.junk stream;
+          let ap_val = parse_token68_or_qstring stream in
+          let rest = parse_auth_param_rest stream in
+          (ap_name, ap_val) :: rest
+      | _ ->
+          []
+
   let parse_credentials mh fn_name fieldname =
     let rec parse_creds stream =
       match Stream.peek stream with 
@@ -1827,32 +1877,7 @@ module Header = struct
 	    let params = parse_auth_params stream in
 	    (auth_name, params)
         | _ ->
-            raise Stream.Failure
-	     
-    and parse_auth_params stream =
-      match Stream.npeek 2 stream with
-	| [ Atom ap_name; Special '=' ] ->
-            Stream.junk stream;
-            Stream.junk stream;
-            let ap_val = parse_token_or_qstring stream in
-            let rest = parse_auth_param_rest stream in
-	    (ap_name, ap_val) :: rest
-        | _ ->
-            raise Stream.Failure
-
-    and parse_auth_param_rest stream =
-      match Stream.npeek 3 stream with
-	| [ Special ','; Atom ap_name; Special '=' ] ->
-            Stream.junk stream;
-            Stream.junk stream;
-            Stream.junk stream;
-            let ap_val = parse_token_or_qstring stream in
-	    let rest = parse_auth_param_rest stream in
-	    (ap_name, ap_val) :: rest
-	| _ ->
-	    []
-    in
-
+            raise Stream.Failure in
     (* Basic authentication is a special case! *)
     let v = mh # field fieldname in  (* or Not_found *)
     match Netstring_str.split ws_re v with
@@ -1863,6 +1888,18 @@ module Header = struct
       | _ ->
 	  parse_field mh fn_name parse_creds fieldname
 
+  let parse_auth_info mh fn_name fieldname =
+    parse_field mh fn_name parse_auth_params fieldname
+
+  let mk_auth_params auth_params =
+    String.concat
+      ","
+      (List.map
+         (fun (p_name, p_val) ->
+           p_name ^ "=" ^ print_param_value p_val)
+         auth_params
+      )
+
   let mk_credentials (auth_name, auth_params) =
     match STRING_LOWERCASE auth_name with
       | "basic"
@@ -1871,15 +1908,10 @@ module Header = struct
 	     try List.assoc "credentials" auth_params 
 	     with Not_found -> 
 	       failwith "Nethttp.mk_credentials: credentials not found" in
-           auth_name ^ " " ^ encode_param creds
+           auth_name ^ " " ^ print_param_value creds
       | _ ->
-           auth_name ^ " " ^ 
-	     (String.concat ","
-	                    (List.map
-	                       (fun (p_name, p_val) ->
-		                p_name ^ "=" ^ encode_param p_val)
-	                       auth_params))
-               
+          auth_name ^ " " ^ mk_auth_params auth_params
+
   let get_authorization mh =
     mark_params_decoded
       (parse_credentials mh "Nethttp.get_authorization" "authorization")
@@ -1895,8 +1927,23 @@ module Header = struct
   let set_proxy_authorization mh v = 
     mh # update_field "Proxy-Authorization" (mk_credentials v)
 
+  let get_authentication_info mh =
+    parse_auth_info
+      mh "Nethttp.get_authentication_infon"
+      "authentication-info"
+    |> List.map mark_decoded
 
+  let get_proxy_authentication_info mh =
+    parse_auth_info
+      mh "Nethttp.get_proxy_authentication_infon"
+      "proxy-authentication-info"
+    |> List.map mark_decoded
 
+  let set_authentication_info mh v =
+    mh # update_field "Authentication-Info" (mk_auth_params v)
+
+  let set_proxy_authentication_info mh v =
+    mh # update_field "Proxy-Authentication-Info" (mk_auth_params v)
 
   (* --- Cookies --- *)
 
