@@ -1,4 +1,4 @@
-(* $Id: netxdr.mli 2195 2015-01-01 12:23:39Z gerd $ *)
+(* $Id$ *)
 
 (** External Data Representation *)
 
@@ -55,6 +55,66 @@
 
 open Netnumber;;
 
+(** Values possible for XDR types. This is straight-forward, except the
+ * "_fast" variants:
+ *)
+
+type xdr_value =
+  | XV_int of int4
+  | XV_uint of uint4
+  | XV_hyper of int8
+  | XV_uhyper of uint8
+  | XV_enum of string
+  | XV_float of fp4
+  | XV_double of fp8
+  | XV_opaque of string
+  | XV_string of string
+  | XV_array of xdr_value array
+  | XV_struct of (string * xdr_value) list
+  | XV_union_over_int of (int4 * xdr_value)
+  | XV_union_over_uint of (uint4 * xdr_value)
+  | XV_union_over_enum of (string * xdr_value)
+  | XV_void
+  (* New in 0.4: *)
+  | XV_enum_fast of int
+      (** The integer is the _position_ in the [X_enum] list, sorted by
+       * enum values (ascending). For example, if we have
+       * [X_enum [ "A", 4; "B", 2; "C", 6 ]]
+       * the element "B" has the position 0, because 2 is the lowest
+       * number
+       *)
+  | XV_struct_fast of xdr_value array
+      (** The array elements are in the same order as declared in [X_struct] *)
+  | XV_union_over_enum_fast of (int * xdr_value)
+      (** The integer is the _position_ in the [X_enum] list. "position"
+       * means the same as for [XV_enum_fast]
+       *)
+  (* New in 3.0: *)
+  | XV_array_of_string_fast of string array
+      (** To be used with an [X_array] or [X_array_fixed] with an inner
+          type of [X_string]
+       *)
+  | XV_mstring of Netxdr_mstring.mstring
+  (* New in 3.5: *)
+  | XV_direct of exn * int * (exn -> xdr_value)
+      (* The variant [XV_direct(exn,size,expand)] should only be used by
+	 code generated with ocamlrpcgen. It is only allowed at positions
+	 in the term with type [X_direct]. It is assumed that the value
+	 is available as exception [exn] (in any form). [size] is the number
+	 of bytes in XDR format. [expand] is a function that can
+         convert the exception to a normal [xdr_value].
+       *) 
+
+  (* TODO: arrays of int, uint, hyper, uhyper, opaque, float, double *)
+
+val xv_true : xdr_value
+val xv_false : xdr_value
+  (** See [x_bool] *)
+
+val xv_none : xdr_value
+val xv_some : xdr_value -> xdr_value
+  (** See [x_optional] *)
+
 (** Terms that describe possible XDR types:
  *
  * - [X_int]:                      integer (32 bit)
@@ -106,7 +166,7 @@ open Netnumber;;
  * XDR unpacker). Values for managed strings are objects of type
  * {!Netxdr_mstring.mstring}.
  *
- * The term [X_direct(t,read,write,size)] is generated by ocamlrpcgen at 
+ * The term [X_direct(t,read,write,size,expand)] is generated by ocamlrpcgen at 
  * positions
  * suitable for direct mapping. In this case, the XDR byte representation is
  * directly mapped to the final Ocaml value bypassing the intermediate
@@ -150,9 +210,10 @@ type xdr_type_term =
   | X_rec of (string * xdr_type_term)      (* define a recursive type *)
   | X_refer of string                      (* refer to a recursive type *)
   | X_direct of xdr_type_term * 
-                (string -> int ref -> int -> exn) *
-                (exn -> string -> int ref -> unit) *
-                (exn -> int)
+                (Bytes.t -> int ref -> int -> exn) *
+                (exn -> Bytes.t -> int ref -> unit) *
+                (exn -> int) *
+                (exn -> xdr_value)
 ;;
 
 type xdr_type;;
@@ -195,65 +256,6 @@ val x_mstring_max : string -> xdr_type_term
 val x_array_max : xdr_type_term -> xdr_type_term
 (** Common abbreviation for arrays of arbitrary length *)
 
-(** Values possible for XDR types. This is straight-forward, except the
- * "_fast" variants:
- *)
-
-type xdr_value =
-  | XV_int of int4
-  | XV_uint of uint4
-  | XV_hyper of int8
-  | XV_uhyper of uint8
-  | XV_enum of string
-  | XV_float of fp4
-  | XV_double of fp8
-  | XV_opaque of string
-  | XV_string of string
-  | XV_array of xdr_value array
-  | XV_struct of (string * xdr_value) list
-  | XV_union_over_int of (int4 * xdr_value)
-  | XV_union_over_uint of (uint4 * xdr_value)
-  | XV_union_over_enum of (string * xdr_value)
-  | XV_void
-  (* New in 0.4: *)
-  | XV_enum_fast of int
-      (** The integer is the _position_ in the [X_enum] list, sorted by
-       * enum values (ascending). For example, if we have
-       * [X_enum [ "A", 4; "B", 2; "C", 6 ]]
-       * the element "B" has the position 0, because 2 is the lowest
-       * number
-       *)
-  | XV_struct_fast of xdr_value array
-      (** The array elements are in the same order as declared in [X_struct] *)
-  | XV_union_over_enum_fast of (int * xdr_value)
-      (** The integer is the _position_ in the [X_enum] list. "position"
-       * means the same as for [XV_enum_fast]
-       *)
-  (* New in 3.0: *)
-  | XV_array_of_string_fast of string array
-      (** To be used with an [X_array] or [X_array_fixed] with an inner
-          type of [X_string]
-       *)
-  | XV_mstring of Netxdr_mstring.mstring
-  (* New in 3.5: *)
-  | XV_direct of exn * int
-      (* The variant [XV_direct(exn,size)] should only be used by
-	 code generated with ocamlrpcgen. It is only allowed at positions
-	 in the term with type [X_direct]. It is assumed that the value
-	 is available as exception [exn] (in any form). [size] is the number
-	 of bytes in XDR format. 
-       *) 
-
-  (* TODO: arrays of int, uint, hyper, uhyper, opaque, float, double *)
-
-val xv_true : xdr_value
-val xv_false : xdr_value
-  (** See [x_bool] *)
-
-val xv_none : xdr_value
-val xv_some : xdr_value -> xdr_value
-  (** See [x_optional] *)
-
 (* Destructors. *)
 
 exception Dest_failure
@@ -403,12 +405,17 @@ val value_matches_type : xdr_value -> xdr_type -> (string*xdr_type) list -> bool
 
 type encoder = Netxdr_mstring.mstring list -> Netxdr_mstring.mstring list
   (** see text above *)
-type decoder = string -> int -> int -> (string * int)
+type decoder = Bytes.t -> int -> int -> (Bytes.t * int)
   (** see text above *)
 
 val pack_xdr_value : ?encode:(string * encoder) list ->
                      xdr_value -> xdr_type -> (string*xdr_type) list ->
-                     (string -> unit) -> unit
+                     (Bytes.t -> unit) -> unit
+val pack_xdr_value_as_bytes :
+                     ?rm:bool ->
+                     ?encode:(string * encoder) list ->
+                     xdr_value -> xdr_type -> (string*xdr_type) list ->
+                       Bytes.t
 val pack_xdr_value_as_string :
                      ?rm:bool ->
                      ?encode:(string * encoder) list ->
@@ -422,6 +429,7 @@ val pack_xdr_value_as_string :
    *)
 
 val pack_xdr_value_as_mstrings :
+       ?rm:bool ->
        ?encode:(string * encoder) list ->
        xdr_value -> xdr_type -> (string*xdr_type) list -> 
          Netxdr_mstring.mstring list
@@ -461,13 +469,13 @@ val unpack_xdr_value : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
                        ?mstring_factories:Netxdr_mstring.named_mstring_factories->
                        ?xv_version:xdr_value_version ->
                        ?decode:(string * decoder) list ->
-                       string -> xdr_type -> (string * xdr_type) list ->
+                       Bytes.t -> xdr_type -> (string * xdr_type) list ->
                        xdr_value
 val unpack_xdr_value_l : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
                        ?mstring_factories:Netxdr_mstring.named_mstring_factories-> 
                          ?xv_version:xdr_value_version ->
                          ?decode:(string * decoder) list ->
-                         string -> xdr_type -> (string * xdr_type) list ->
+                         Bytes.t -> xdr_type -> (string * xdr_type) list ->
                          (xdr_value * int)
   (** [prefix]: whether it is ok that the string is longer than the message
    *   (default: false)
@@ -492,6 +500,15 @@ val unpack_xdr_value_l : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
    * be raised.
    *)
 
+val unpack_xdr_value_str : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
+                       ?mstring_factories:Netxdr_mstring.named_mstring_factories-> 
+                         ?xv_version:xdr_value_version ->
+                         ?decode:(string * decoder) list ->
+                         string -> xdr_type -> (string * xdr_type) list ->
+                         (xdr_value * int)
+  (** Same for strings *)
+
+
 (**/**)
 
 (* Low-level *)
@@ -506,12 +523,12 @@ val sizefn_string : Netnumber.uint4 -> string -> int
 val sizefn_mstring : Netnumber.uint4 -> Netxdr_mstring.mstring -> int
   (* Return the packed size of an mstring *)
 
-val write_string_fixed : int -> string -> string -> int ref -> unit
-val write_string : string -> string -> int ref -> unit
+val write_string_fixed : int -> string -> Bytes.t -> int ref -> unit
+val write_string : string -> Bytes.t -> int ref -> unit
   (* [write_* s1 s2 p]: write the string s1 as XDR to s2 at position p *)
 
-val read_string_fixed : int -> string -> int ref -> int -> string
-val read_string : Netnumber.uint4 -> string -> int ref -> int -> string
+val read_string_fixed : int -> Bytes.t -> int ref -> int -> string
+val read_string : Netnumber.uint4 -> Bytes.t -> int ref -> int -> string
 
 val raise_xdr_format_too_short : unit -> 'a
 val raise_xdr_format_value_not_included : unit -> 'a
@@ -520,3 +537,13 @@ val raise_xdr_format_undefined_descriminator : unit -> 'a
 
 val safe_add : int -> int -> int
 val safe_mul : int -> int -> int
+
+type ctx =
+  { ctx_direct : bool;
+    ctx_direct_sub : bool;
+    ctx_copy_string : string -> string
+  }
+
+val default_ctx : ctx
+val expand_ctx : ctx
+val direct_ctx : ctx
